/*  pmanage.h: (c) 2002 sibn

    This file is part of GWiz.

    GWiz is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    GWiz is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GWiz; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* These are included here to ensure that the correct header files are included
   with pmanage.c. They can be safely moved to pmanage.c BEFORE the #include
   "pmanage.h" directive if you prefer. */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <stdarg.h>

#ifndef HAVE_PMANAGE_H
#define HAVE_PMANAGE_H

/* useful defines.  Use these when setting preferences. */
#define FMT_STRING 's'
#define FMT_INT    'd'
#define FMT_DOUBLE 'f'

typedef struct INI_ INI;
typedef struct PSect_ PSect;
typedef struct PNode_ PNode;

struct PNode_ {
    PNode *next;
    char *line;
};

struct PSect_ {
    PSect *next;
    PNode *node;
    char *name;
};

struct INI_ {
    PSect *list;
    PSect *dlist; /* default options to use if requested one does not exist */
    FILE *file;
    FILE *dfile; /* file default options come from if option does not exist */
    char *path;
    void (*error_handle)(void);
};

/* NewINI is used to create a new INI instance.  Generally, you create an INI
   when your program runs, and leave it open until the program closes (and
   optionally calling SaveINI() periodically to record changes).  "path" is the
   path to the ini file you wish to open, "dpath" is the path to the default
   ini file.  If you request an option that does not exist, then pmanage will
   check the default config as well, to see if it's there (returning NULL or 0
   if it is not. */
INI *NewINI (char *path, char *dpath, void (*error_handle)(void));

/* Save an INI.  This will NOT close the ini.  To do that, you must manually
   call DestroyINI(). */
void SaveINI (INI *ini);

/* Force INI file to clean up and exit.  THIS WILL NOT SAVE CHANGES. */
void DestroyINI (INI *ini);

/* Get a setting name (in the form of a char*) */
char *GetSetting (INI *ini, char *section, char *oname);

/* Get a setting value (true or false).  false = 0, true = 1. */
short GetSettingBool (INI *ini, char *section, char *oname);

/* Get a setting value (integer).  Value is signed. */
int GetSettingInt (INI *ini, char *section, char *oname);

/* Get a setting value (float).  Value is signed. */
float GetSettingFloat (INI *ini, char *section, char *oname);

/* Get a setting value (double).  Value is signed */
double GetSettingDouble (INI *ini, char *section, char *oname);

/* Change a setting.  Usage is: 
   SetSetting (ini, "video", "fullscreen", FMT_STRING, "true");
   Note the importance of setting a boolean value with a char* instead of an
   int or a short.  Booleans can be true/false/yes/no/y/n/1/0.  All are
   treated equally. */ 
void SetSetting (INI *ini, char *section, char *oname, char fmt, void *val);




/* Internal use only below this line: */
void DestroyPSects (INI *ini);

PSect *NewPSect (INI *ini, char *assignment);

PNode *NewPNode (INI *ini, char *assignment);

PNode *AppendPNode (INI *ini, char *section, char *assignment);

PSect *AppendPSect (INI *ini, char *assignment);

char *ReadINILine (INI *ini);

void AppendDPNode (INI *ini, char *section, char *assignment);

void AppendDPSect (INI *ini, char *assignment);

char *ReadDINILine (INI *ini);

char *GetDefaultSetting(INI *ini, char *section, char *oname);

PSect *GetPSect (INI *ini, char *sname);

PNode *GetPNode (INI *ini, PSect *sect, char *oname);

void errhandle(void);

#endif /* HAVE_PMANAGE_H */
